﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace TwoArraysWithoutIteratorMethods {

	class A : IEnumerable<int> {
		private int[] x1 = { 1, 2 };
		private int[] x2 = { 3, 4, 5 };

		System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator() {
			return GetEnumerator();
		}

		public IEnumerator<int> GetEnumerator() {
			return new Enumerator(this);
		}

		private class Enumerator : IEnumerator<int> {
			private A a;
			private int index = -1;
			private bool firstArray = true;

			public Enumerator(A a) {
				this.a = a;
			}

			public int Current {
				get {
					if (firstArray) {
						if (index < 0) {
							throw new InvalidOperationException("Enumeration has not started. Call MoveNext.");
						} else {
							return a.x1[index];
						}
					} else {
						if (index >= a.x2.Length) {
							throw new InvalidOperationException("Enumeration already finished.");
						} else {
							return a.x2[index];
						}
					}
				}
			}

			public bool MoveNext() {
				if (firstArray) {
					if (index < a.x1.Length - 1) {
						index++;
						return true;
					} else {
						index = -1;
						firstArray = false;
					}
				}

				if (index < a.x2.Length - 1) {
					index++;
					return true;
				} else {
					if (index < a.x2.Length) {
						index++;
					}
					return false;
				}

			}

			public void Reset() {
				index = -1;
				firstArray = true;
			}

			object System.Collections.IEnumerator.Current {
				get { return Current; }
			}

			public void Dispose() {
			}
		}
	}

	class Program {
		static void Main(string[] args) {
			int[] ints = { 10, 20 };

			var e = ints.GetEnumerator();
			try {
				Console.WriteLine("on int[]: before while: e.Current == {0}", e.Current);
			} catch (Exception ex) {
				Console.WriteLine("{0}: {1}", ex.GetType(), ex.Message);
			}
			while (e.MoveNext()) {
				Console.WriteLine("on int[]: in while: e.Current == {0}", e.Current);
			}
			try {
				Console.WriteLine("on int[]: after while: e.Current == {0}", e.Current);
			} catch (Exception ex) {
				Console.WriteLine("{0}: {1}", ex.GetType(), ex.Message);
			}

			//
			Console.WriteLine();
			//

			A a = new A();
			Console.WriteLine("on A: before foreach");
			foreach (int i in a) {
				Console.WriteLine("on A: got {0}", i);
			}
			Console.WriteLine("on A: after foreach");

			//
			Console.WriteLine();
			//

			e = a.GetEnumerator();
			try {
				Console.WriteLine("on A: before while: e.Current == {0}", e.Current);
			} catch (Exception ex) {
				Console.WriteLine("{0}: {1}", ex.GetType(), ex.Message);
			}
			while (e.MoveNext()) {
				Console.WriteLine("on A: in while: e.Current == {0}", e.Current);
			}
			try {
				Console.WriteLine("on A: after while: e.Current == {0}", e.Current);
			} catch (Exception ex) {
				Console.WriteLine("{0}: {1}", ex.GetType(), ex.Message);
			}
		}
	}
}
